-- noqa: disable=all
-- This sql was autogenerated as a reference example using the library
-- CLI. Its format is tied to the specific database it was run against,
-- and it may not be correct for all databases. Use the CLI's build 
-- option to derive the best SQL for your dataset.

-- ###########################################################

CREATE TABLE discovery__code_sources AS
--noqa: disable=LTO2,LT09,CV06
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_allergyintolerance_clinicalstatus
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_allergyintolerance_verificationstatus
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_allergyintolerance_code
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_allergyintolerance_reaction_substance
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_allergyintolerance_reaction_manifestation
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_allergyintolerance_reaction_exposureroute
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_condition_category
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_condition_code
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_diagnosticreport_category
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_diagnosticreport_code
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_diagnosticreport_conclusioncode
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_documentreference_type
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_documentreference_category
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_encounter_class
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_encounter_type
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_encounter_servicetype
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_encounter_priority
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_encounter_reasoncode
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_encounter_hospitalization_dischargedisposition
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_location_type
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_medication_codecodeableconcept
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_medication_medicationcode
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_medicationrequest_medicationcodeableconcept
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_observation_category
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_observation_code
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_observation_component_code
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_observation_interpretation
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_observation_valuecodeableconcept
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_observation_dataabsentreason
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_organization_type
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_patient_maritalstatus
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_practitioner_qualification_code
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_practitionerrole_code
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_practitionerrole_specialty
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_procedure_statusreason
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_procedure_category
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_procedure_code
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_procedure_performer_function
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_procedure_reasoncode
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_procedure_bodysite
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_procedure_outcome
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_procedure_complication
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_procedure_followup
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_procedure_focaldevice_action
UNION ALL
SELECT
    table_name,
    column_name,
    code,
    display,
    system
FROM discovery__tmp_procedure_usedcode
;
